using System.Numerics;

namespace FastRng.Distributions;

public sealed class CauchyLorentzX0<TNum> : Distribution<TNum> where TNum : IFloatingPointIeee754<TNum>
{
    private static readonly TNum CONSTANT = TNum.CreateChecked(0.31f);
    private static readonly TNum SCALE = TNum.CreateChecked(0.1f);
    private static readonly TNum MEDIAN = TNum.Zero;
    private static readonly TNum TWO = TNum.CreateChecked(2f);

    public CauchyLorentzX0(IRandom<TNum> rng) : base(rng)
    {
    }

    /// <inheritdoc />
    public override TNum ShapeFunction(TNum x) => CONSTANT * ( TNum.One / (TNum.Pi * SCALE)) * ((SCALE * SCALE) / (TNum.Pow(x - MEDIAN, TWO) + (SCALE * SCALE)));
}