using System.Numerics;

namespace FastRng.Distributions;

public sealed class GammaA5B15<TNum> : Distribution<TNum> where TNum : IFloatingPointIeee754<TNum>
{
    private static readonly TNum ALPHA = TNum.CreateChecked(5.0f);
    private static readonly TNum BETA = TNum.CreateChecked(15.0f);
    private static readonly TNum CONSTANT = TNum.CreateChecked(0.341344210715475f);

    private static readonly TNum GAMMA_ALPHA;
    private static readonly TNum BETA_TO_THE_ALPHA;
        
    static GammaA5B15()
    {
        GAMMA_ALPHA = MathToolsFloatingPoint<TNum>.Gamma(ALPHA);
        BETA_TO_THE_ALPHA = TNum.Pow(BETA, ALPHA);
    }

    public GammaA5B15(IRandom<TNum> rng) : base(rng)
    {
    }

    /// <inheritdoc />
    public override TNum ShapeFunction(TNum x) => CONSTANT * ((BETA_TO_THE_ALPHA * TNum.Pow(x, ALPHA - TNum.One) * TNum.Exp(-BETA * x)) / GAMMA_ALPHA);
}