using System.Numerics;

namespace FastRng.Distributions;

public sealed class InverseExponentialLa5<TNum> : Distribution<TNum> where TNum : IFloatingPointIeee754<TNum>
{
    private static readonly TNum LAMBDA = TNum.CreateChecked(5.0f);
    private static readonly TNum CONSTANT = TNum.CreateChecked(0.001347589399817f);

    public InverseExponentialLa5(IRandom<TNum> rng) : base(rng)
    {
    }

    /// <inheritdoc />
    public override TNum ShapeFunction(TNum x) => CONSTANT * LAMBDA * TNum.Exp(LAMBDA * x);
}