using System.Numerics;

namespace FastRng.Distributions;

public sealed class InverseGammaA3B05<TNum> : Distribution<TNum> where TNum : IFloatingPointIeee754<TNum>
{
    private static readonly TNum ALPHA = TNum.CreateChecked(3.0f);
    private static readonly TNum BETA = TNum.CreateChecked(0.5f);
    private static readonly TNum CONSTANT = TNum.CreateChecked(0.213922656884911f);

    private static readonly TNum FACTOR_LEFT;
        
    static InverseGammaA3B05()
    {
        var gammaAlpha = MathToolsFloatingPoint<TNum>.Gamma(ALPHA);
        var betaToTheAlpha = TNum.Pow(BETA, ALPHA);
            
        FACTOR_LEFT = CONSTANT * (betaToTheAlpha / gammaAlpha);
    }
        
    public InverseGammaA3B05(IRandom<TNum> rng) : base(rng)
    {
    }

    /// <inheritdoc />
    public override TNum ShapeFunction(TNum x) => FACTOR_LEFT * TNum.Pow(x, -ALPHA - TNum.One) * TNum.Exp(-BETA / x);
}