using System.Numerics;

namespace FastRng.Distributions;

public sealed class LaplaceB01M05<TNum> : Distribution<TNum> where TNum : IFloatingPointIeee754<TNum>
{
    private static readonly TNum TWO = TNum.CreateChecked(2.0f);
    private static readonly TNum B = TNum.CreateChecked(0.1f);
    private static readonly TNum MU = TNum.CreateChecked(0.5f);
    private static readonly TNum CONSTANT = TNum.CreateChecked(0.2f);
        
    private static readonly TNum FACTOR_LEFT;
        
    static LaplaceB01M05()
    {
        FACTOR_LEFT = CONSTANT / (TWO * B);
    }
        
    public LaplaceB01M05(IRandom<TNum> rng) : base(rng)
    {
    }

    /// <inheritdoc />
    public override TNum ShapeFunction(TNum x) => FACTOR_LEFT * TNum.Exp(-TNum.Abs(x - MU) / B);
}