using System.Numerics;

namespace FastRng.Distributions;

public sealed class LogNormalS1M0<TNum> : Distribution<TNum> where TNum : IFloatingPointIeee754<TNum>
{
    private static readonly TNum TWO = TNum.CreateChecked(2f);
    private static readonly TNum SIGMA = TNum.One;
    private static readonly TNum MU = TNum.Zero;
    private static readonly TNum CONSTANT = TNum.CreateChecked(1.51998658387455f);
        
    private static readonly TNum FACTOR;
        
    static LogNormalS1M0()
    {
        FACTOR = SIGMA * TNum.Sqrt(TWO * TNum.Pi);
    }
        
    public LogNormalS1M0(IRandom<TNum> rng) : base(rng)
    {
    }

    /// <inheritdoc />
    public override TNum ShapeFunction(TNum x) => (CONSTANT / (x * FACTOR)) * TNum.Exp( -(TNum.Pow(TNum.Log(x) - MU, TWO) / (TWO * TNum.Pow(SIGMA, TWO))));
}