using System.Numerics;

namespace FastRng.Distributions;

public sealed class WeibullK05La1<TNum> : Distribution<TNum> where TNum : IFloatingPointIeee754<TNum>
{
    private static readonly TNum K = TNum.CreateChecked(0.5f);
    private static readonly TNum LAMBDA = TNum.One;
    private static readonly TNum CONSTANT = TNum.CreateChecked(0.221034183615129f);

    public WeibullK05La1(IRandom<TNum> rng) : base(rng)
    {
    }

    /// <inheritdoc />
    public override TNum ShapeFunction(TNum x) => CONSTANT * ( (K / LAMBDA) * TNum.Pow(x / LAMBDA, K -  TNum.One) * TNum.Exp(-TNum.Pow(x/LAMBDA, K)));
}